'This program runs the following sequence:
'1. Homing with input 1.
'2. Move to the "Spray Position".
'3. Start a timer.
'4. Wait for the timer to finish.
'5. Turn on nozzles and start moving back to home.
'6. Turn off nozzles after specified distance.
'The program contains parameter settings for the units, mm, mm/s, and mm/s^2.
'All variables related to position, speed, and acceleration must be in integer format.
'The program contains interrupts for conditions and restarts the program when needed.
'The program works in conjunction with an HMI communicating over Modbus.
'-------------- Device Params -----------------------
Params
UNIT.PROTARY = 3 'custom
UNIT.PIN = 381 '381mm/10revs is for 2:1 ratio and 3"/rev belt drive
UNIT.POUT = 10
UNIT.ACCROTARY = 3 'mm/s^2
UNIT.VROTARY = 3 'mm/s
UNIT.LABEL = "mm"
End Params

'-------------- Define (dim) Global Variables --------
Dim starthome as integer
Dim homefound as integer
Dim filmsize as integer 'distance to spray on return stroke, in mm
Dim filmspeed as integer 'speed of spray operation, in mm/s
Dim dissolvetime as integer 'in seconds
Dim nozzle1 as integer 'nozzle #1 checkbox
Dim nozzle2 as integer 'nozzle #2 checkbox
Dim startbutton as integer
Dim timemark as integer 'in milliseconds
Dim sprayposition as integer 'in mm
Dim speed1 as integer 'speed of initial move to sprayposition
Dim accel as integer 'acceleration for all moves
Dim decel as integer 'deceleration for all moves
Dim countdown as integer 'status indicator for countdown of the timer
Dim ready as integer 'status indicator for spraying mode


MBInfo
$MBMap32(5000, homefound) 'Modbus registers 5000 and 5001 for homefound status to HMI
$MBMap32(5002, filmsize) 'Modbus registers 5002 and 5003 for filmsize from HMI
$MBMap32(5004, filmspeed) 'Modbus registers 5004 and 5005 for filmspeed from HMI
$MBMap32(5006, dissolvetime) 'Modbus registers 5006 and 5007 for dissolvetime from HMI
$MBMap32(5008, nozzle1) 'Modbus registers 5008 and 5009 for nozzle1 from HMI
$MBMap32(5010, nozzle2) 'Modbus registers 5010 and 5011 for nozzle2 from HMI
$MBMap32(5012, startbutton) 'Modbus registers 5012 and 5013 for start from HMI
$MBMap32(5014, starthome) 'Modbus registers 5014 and 5015 for starthome from HMI
$MBMap32(5016, countdown) 'Modbus registers 5016 and 5017 for countdown status to HMI
$MBMap32(5018, ready) 'Modbus registers 5018 and 5019 for spraying status to HMI
End
'-------------- Main Program -------------------------
Main
DRV.SWENABLE = 0
starthome = 0
homefound = 0
startbutton = 0

'Enable Interrupts
INTR.HWLS.POSSTATE = 1
INTR.HWLS.NEGSTATE = 1
Intr.DRV.FAULTS = 1
INTR.DISABLE = 1
INTR.DRV.HWENABLE = 1

'Set default values
sprayposition = 1000 'total distance, in mm
speed1 = 200 'speed of initial move to sprayposition, in mm/s
accel = 1000 'mm/s^2
decel = 1000 'mm/s^2
dissolvetime = 8 'in seconds
filmsize = 800 'distance to spray on return stroke, in mm
filmspeed = 200 'speed of spray operationm in mm/s
nozzle1 = 0 'nozzle #1 checkbox
nozzle2 = 0 'nozzle #2 checkbox

Move.Acc = accel  ' Acceleration (drive units)
Move.Dec = decel  ' Deceleration (drive units)
DOUT1.STATEU = 0 'turn off nozzle1
DOUT2.STATEU = 0 'turn off nozzle2

While DRV.ACTIVE = 0 : Wend 'wait for hardware and software enable

While 1 = 1 'This loops the rest of the program.

'Homing: uses Input 1 for the position capture
If homefound = 0 Then 'If axis has not been homed, then wait for starthome button
	While starthome = 0 : Wend 'start the homing routine when starthome = 1
	startbutton = 0 'reset startbutton to zero in case it was pressed prior to homing
	CAP0.MODE = 0 'standard position capture
	CAP0.TRIGGER = 0 'Use input 1 for position capture
	CAP0.EDGE = 1 'positive edge
	CAP0.EVENT = 0 'no pre-conditions must be met
	CAP0.EN = 1 'Start looking for the capture trigger
	MOVE.RUNSPEED = 200 'mm/s
	MOVE.DIR = 1 'negative direction
	MOVE.GOVEL 'start moving
	While CAP0.STATE = 0 : Wend 'wait for the capture to trigger
	'Print "Home input triggered"
	MOVE.RUNSPEED = 0
	MOVE.GOVEL 'stop moving, zero speed
	While MOVE.MOVING = 1 : Wend 'wait for motion to stop
	MOVE.POSCOMMAND = (PL.FB -CAP0.PLFB ) 'set position to the difference between the actual position and captured position
	Pause (.2)
	MOVE.RUNSPEED = 50 'mm/s
	MOVE.TARGETPOS = 0
	MOVE.GOABS 'move back to the captured position
	Pause (1) 'this is the end of the homing routine
	starthome = 0 'reset starthome trigger
	homefound = 1 'set homefound tag
End If

While DRV.HWENABLE = 0  : Wend 'Wait for the hardware enable to be turned on
While startbutton = 0 : Wend 'Wait for startbutton = 1 from HMI
Move.RunSpeed = speed1 'mm/s, speed of initial move
Move.TargetPos = sprayposition 'mm, target position is the position at which the sprayers will start
Move.GoAbs 'start initial move
While MOVE.MOVING = 1 : Wend 'wait for motion to stop
timemark = DRV.TIME 'record runtime of drive in milliseconds
countdown = 1 'turn on status indicator for countdown
While DRV.TIME < dissolvetime * 1000 + timemark - 1000 : Wend 'wait for timer minus 1 sec
countdown = 0 'turn off status indicator for countdown
ready = 1 'turn on ready status indicator
Move.RunSpeed   = filmspeed  'mm/s
Move.TargetPos  = 0   'mm, home position
Pause (1)
ready = 0 'turn off ready status indicator
Move.GoAbs 'start return move for spray operation
DOUT1.STATEU = nozzle1 'turn on nozzle1
DOUT2.STATEU = nozzle2 'turn on nozzle2
If sprayposition - filmsize > 0 Then 'make sure the film size is less than total move distance
	While PL.FB > sprayposition - filmsize : Wend 'position to turn off nozzles
ElseIf sprayposition - filmsize < 0 Then
	While PL.FB > 1 : Wend 'turn off nozzles just before reaching home position
End If
DOUT1.STATEU = 0 'turn off nozzle1
DOUT2.STATEU = 0 'turn off nozzle2
startbutton = 0 'reset start trigger
Wend
End Main

'-------------- Subroutines and Functions ------------

'-------------- Interrupt Routines -------------------
Interrupt HWLS.NEGSTATE
	If homefound = 0 Then
	While MOVE.MOVING = 1 : Wend
		MOVE.DIR = 0 'positive direction
		MOVE.RUNSPEED = 50 'mm/s
		MOVE.GOVEL 'start moving
		While DIN4.STATE = 0 : Wend
		While DIN4.STATE = 1 : Wend
		Pause (0.5)
		MOVE.RUNSPEED = 0
		MOVE.GOVEL 'stop moving, zero speed
		While MOVE.MOVING = 1 : Wend
		CAP0.EN = 1 'Start looking for the capture trigger
		MOVE.DIR = 1 'negative direction
		MOVE.RUNSPEED = 50 'mm/s
		MOVE.GOVEL 'start moving
	ElseIf homefound = 1 Then
		Restart
	End If
    INTR.HWLS.NEGSTATE = 1
End Interrupt

Interrupt HWLS.POSSTATE
	Restart
End Interrupt

Interrupt DRV.HWENABLE
	Restart
    Intr.DRV.HWENABLE = 1
End Interrupt

Interrupt disable
	Restart
	INTR.DISABLE = 1
End Interrupt

Interrupt DRV.FAULTS
	Restart
End Interrupt